@@ -18,6 +18,14 @@ type PolicyBuilder struct {
 	fHandler *utils.FilesHandler
 }
 
+// struct used to keep the user PGT sourceFile data with the actual built CR
+// from that source file.
+type generatedCR struct {
+	globalComplianceType string
+	pgtSourceFile        utils.SourceFile
+	builtCR              map[string]interface{}
+}
+
 func NewPolicyBuilder(fileHandler *utils.FilesHandler) *PolicyBuilder {
 	return &PolicyBuilder{fHandler: fileHandler}
 }
@@ -67,9 +75,19 @@ func (pbuilder *PolicyBuilder) Build(policyGenTemp utils.PolicyGenTemplate) (map
 				name := strings.Join([]string{policyGenTemp.Metadata.Name, sFile.PolicyName}, "-")
 				output := path.Join(policyGenTemp.Metadata.Name, name)
 				var acmPolicy utils.AcmPolicy
+				// resources is a map of only the built CRs. Bind them with sourceFile
+				// data as needed by low level methods setting various policy attributes
+				annotatedResources := make([]generatedCR, len(resources))
+				for idx, cr := range resources {
+					annotatedResources[idx] = generatedCR{
+						globalComplianceType: policyGenTemp.Spec.ComplianceType,
+						pgtSourceFile:        sFile,
+						builtCR:              cr,
+					}
+				}
 				if sFile.PolicyName != "" && policies[output] == nil {
 					// Generate new policy
-					acmPolicy, err = pbuilder.createAcmPolicy(name, policyGenTemp.Metadata.Namespace, resources)
+					acmPolicy, err = pbuilder.createAcmPolicy(name, policyGenTemp.Metadata.Namespace, annotatedResources)
 					if err != nil {
 						return policies, err
 					}
@@ -77,7 +95,7 @@ func (pbuilder *PolicyBuilder) Build(policyGenTemp utils.PolicyGenTemplate) (map
 					subjects = append(subjects, subject)
 				} else if sFile.PolicyName != "" && policies[output] != nil {
 					// Append new CR to policy
-					acmPolicy, err = pbuilder.AppendAcmPolicy(policies[output].(utils.AcmPolicy), resources)
+					acmPolicy, err = pbuilder.AppendAcmPolicy(policies[output].(utils.AcmPolicy), annotatedResources)
 					if err != nil {
 						return policies, err
 					}
@@ -164,9 +182,19 @@ func (pbuilder *PolicyBuilder) getCustomResource(sourceFile utils.SourceFile, so
 	}
 	if resourceMap["spec"] != nil {
 		resourceMap["spec"] = pbuilder.setValues(resourceMap["spec"].(map[string]interface{}), sourceFile.Spec)
+	} else if sourceFile.Spec != nil {
+		// If the user supplies a "spec" section but the source CR does not have
+		// one, this will ensure we pull in the user content
+		resourceMap["spec"] = make(map[string]interface{})
+		resourceMap["spec"] = pbuilder.setValues(resourceMap["spec"].(map[string]interface{}), sourceFile.Spec)
 	}
 	if resourceMap["data"] != nil {
 		resourceMap["data"] = pbuilder.setValues(resourceMap["data"].(map[string]interface{}), sourceFile.Data)
+	} else if sourceFile.Data != nil {
+		// If the user supplies a "data" section but the source CR does not have
+		// one, this will ensure we pull in the user content
+		resourceMap["data"] = make(map[string]interface{})
+		resourceMap["data"] = pbuilder.setValues(resourceMap["data"].(map[string]interface{}), sourceFile.Data)
 	}
 
 	return resourceMap, nil
@@ -205,6 +233,17 @@ func (pbuilder *PolicyBuilder) setValues(sourceMap map[string]interface{}, value
 			sourceMap[k] = valueMap[k]
 		}
 	}
+
+	// If the user provides values in the PGT spec/data sections for which there
+	// is not a corresponding key in the source-cr we need to simply copy over
+	// the user supplied values
+	for k, v := range valueMap {
+		// Merging user content into fields that are already in the source-cr is
+		// handled in previous loop
+		if sourceMap[k] == nil {
+			sourceMap[k] = v
+		}
+	}
 	return sourceMap
 }
 
@@ -232,27 +271,36 @@ func (pbuilder *PolicyBuilder) splitYamls(yamls []byte) ([][]byte, error) {
 	return resources, nil
 }
 
-func (pbuilder *PolicyBuilder) readAcmPolicyTemplate() (utils.AcmPolicy, error) {
+var (
+	// Contents of ACM policy template file. Cached for performance.
+	acmPolicyTemplate []byte
+)
+
+// generate a new AcmPolicy struct from reference ACM policy template
+func (pbuilder *PolicyBuilder) buildAcmPolicyObject() (utils.AcmPolicy, error) {
 	acmPolicy := utils.AcmPolicy{}
-	acmPolicyTemp, err := pbuilder.fHandler.ReadResourceFile(utils.ACMPolicyTemplate)
-	if err != nil {
-		return acmPolicy, err
-	}
-	err = yaml.Unmarshal(acmPolicyTemp, &acmPolicy)
 
-	if err != nil {
-		return acmPolicy, err
+	// Singleton for the template byte array so we only have to read it once
+	if len(acmPolicyTemplate) == 0 {
+		acmPolicyTemp, err := pbuilder.fHandler.ReadResourceFile(utils.ACMPolicyTemplate)
+		if err != nil {
+			return acmPolicy, err
+		}
+		acmPolicyTemplate = make([]byte, len(acmPolicyTemp))
+		copy(acmPolicyTemplate, acmPolicyTemp)
 	}
 
-	return acmPolicy, nil
+	err := yaml.Unmarshal(acmPolicyTemplate, &acmPolicy)
+
+	return acmPolicy, err
 }
 
-func (pbuilder *PolicyBuilder) createAcmPolicy(name string, namespace string, resources []map[string]interface{}) (utils.AcmPolicy, error) {
+func (pbuilder *PolicyBuilder) createAcmPolicy(name string, namespace string, resources []generatedCR) (utils.AcmPolicy, error) {
 	if err := CheckNameLength(namespace, name); err != nil {
 		return utils.AcmPolicy{}, err
 	}
 
-	acmPolicy, err := pbuilder.readAcmPolicyTemplate()
+	acmPolicy, err := pbuilder.buildAcmPolicyObject()
 	if err != nil {
 		return acmPolicy, err
 	}
@@ -270,9 +318,7 @@ func (pbuilder *PolicyBuilder) createAcmPolicy(name string, namespace string, re
 	objTempArr := make([]utils.ObjectTemplates, len(resources))
 
 	for idx, resource := range resources {
-		objTemp := utils.ObjectTemplates{}
-		objTemp.ComplianceType = acmPolicy.Spec.PolicyTemplates[0].ObjDef.Spec.ObjectTemplates[0].ComplianceType
-		objTemp.ObjectDefinition = resource
+		objTemp := BuildObjectTemplate(resource)
 		objTempArr[idx] = objTemp
 	}
 	acmPolicy.Spec.PolicyTemplates[0].ObjDef.Spec.ObjectTemplates = objTempArr
@@ -280,7 +326,7 @@ func (pbuilder *PolicyBuilder) createAcmPolicy(name string, namespace string, re
 	return acmPolicy, nil
 }
 
-func (pbuilder *PolicyBuilder) AppendAcmPolicy(acmPolicy utils.AcmPolicy, resources []map[string]interface{}) (utils.AcmPolicy, error) {
+func (pbuilder *PolicyBuilder) AppendAcmPolicy(acmPolicy utils.AcmPolicy, resources []generatedCR) (utils.AcmPolicy, error) {
 	if len(acmPolicy.Spec.PolicyTemplates) < 1 {
 		return acmPolicy, errors.New("Mising Policy template in the " + acmPolicy.Metadata.Name)
 	}
@@ -291,9 +337,7 @@ func (pbuilder *PolicyBuilder) AppendAcmPolicy(acmPolicy utils.AcmPolicy, resour
 	objTempArr := acmPolicy.Spec.PolicyTemplates[0].ObjDef.Spec.ObjectTemplates
 
 	for _, resource := range resources {
-		objTemp := utils.ObjectTemplates{}
-		objTemp.ComplianceType = acmPolicy.Spec.PolicyTemplates[0].ObjDef.Spec.ObjectTemplates[0].ComplianceType
-		objTemp.ObjectDefinition = resource
+		objTemp := BuildObjectTemplate(resource)
 		objTempArr = append(objTempArr, objTemp)
 	}
 	acmPolicy.Spec.PolicyTemplates[0].ObjDef.Spec.ObjectTemplates = objTempArr
